@@ -71,6 +71,8 @@ static const cmdline_option_t g_tboot_cmdline_options[] = {
     { "serial",     "115200,8n1,0x3f8" },
     /* serial=<baud>[/<clock_hz>][,<DPS>[,<io-base>[,<irq>[,<serial-bdf>[,<bridge-bdf>]]]]] */
     { "vga_delay",  "0" },           /* # secs */
+    { "ap_wake_mwait", "false" },    /* true|false */
+    { "pcr_map", "legacy" },         /* legacy|da */
     { NULL, NULL }
 };
 static char g_tboot_param_values[ARRAY_SIZE(g_tboot_cmdline_options)][MAX_VALUE_LEN];
@@ -108,7 +110,7 @@ static const char* get_option_val(const cmdline_option_t *options,
     return NULL;
 }
 
-static void cmdline_parse(char *cmdline, const cmdline_option_t *options,
+static void cmdline_parse(const char *cmdline, const cmdline_option_t *options,
                           char vals[][MAX_VALUE_LEN])
 {
     const char *p = cmdline;
@@ -168,7 +170,7 @@ void tboot_parse_cmdline(void)
     cmdline_parse(g_cmdline, g_tboot_cmdline_options, g_tboot_param_values);
 }
 
-void linux_parse_cmdline(char *cmdline)
+void linux_parse_cmdline(const char *cmdline)
 {
     cmdline_parse(cmdline, g_linux_cmdline_options, g_linux_param_values);
 }
@@ -430,11 +432,29 @@ void get_tboot_vga_delay(void)
     g_vga_delay = strtoul(vga_delay, NULL, 0);
 }
 
+bool get_tboot_prefer_da(void)
+{
+    const char *value = get_option_val(g_tboot_cmdline_options,
+                                       g_tboot_param_values, "pcr_map");
+    if ( value != NULL && strcmp(value, "da") == 0 )
+        return true;
+
+    return false;
+}
 
 /*
  * linux kernel command line parsing
  */
 
+bool get_tboot_mwait(void)
+{
+    const char *mwait = get_option_val(g_tboot_cmdline_options,
+                                       g_tboot_param_values, "ap_wake_mwait");
+    if ( mwait == NULL || strcmp(mwait, "false") == 0 )
+        return false;
+    return true;
+}
+
 bool get_linux_vga(int *vid_mode)
 {
     const char *vga = get_option_val(g_linux_cmdline_options,
