@@ -53,9 +53,15 @@
 #include <uuid.h>
 #include <loader.h>
 #include <hash.h>
+#include <mle.h>
 #include <tb_error.h>
 #include <txt/txt.h>
 #include <txt/vmcs.h>
+#include <txt/smx.h>
+#include <txt/mtrrs.h>
+#include <txt/config_regs.h>
+#include <txt/heap.h>
+#include <txt/verify.h>
 #include <tb_policy.h>
 #include <tboot.h>
 #include <acpi.h>
@@ -74,8 +80,6 @@ void s3_launch(void);
 
 extern long s3_flag;
 
-extern char _start[];            /* start of tboot */
-extern char _end[];              /* end of tboot */
 extern char s3_wakeup_16[];
 extern char s3_wakeup_end[];
 
@@ -95,9 +99,9 @@ extern tboot_shared_t _tboot_shared;
  */
 static __data uint8_t g_saved_s3_wakeup_page[PAGE_SIZE];
 
-void *get_tboot_mem_end(void)
+unsigned long get_tboot_mem_end(void)
 {
-    return (void *)(((unsigned long)&_end + PAGE_SIZE - 1ULL) & ~(PAGE_SIZE - 1));
+    return PAGE_UP((unsigned long)&_end);
 }
 
 static tb_error_t verify_platform(void)
@@ -147,8 +151,7 @@ static void post_launch(void)
 
     printk("measured launch succeeded\n");
 
-    err = txt_post_launch();
-    apply_policy(err);
+    txt_post_launch();
 
     /* backup DMAR table */
     save_vtd_dmar_table();
@@ -172,7 +175,7 @@ static void post_launch(void)
     apply_policy(err);
 
     /* verify that tboot is in valid RAM (i.e. E820_RAM) */
-    base = (uint64_t)((unsigned long)&_start - 3*PAGE_SIZE);
+    base = (uint64_t)TBOOT_BASE_ADDR;
     size = (uint64_t)((unsigned long)&_end - base);
     printk("verifying tboot and its page table (%Lx - %Lx) in e820 table\n\t",
            base, (base + size - 1));
@@ -184,8 +187,8 @@ static void post_launch(void)
         printk(": succeeded.\n");
 
     /* protect ourselves, MLE page table, and MLE/kernel shared page */
-    base = (uint64_t)((unsigned long)&_start - 3*PAGE_SIZE);
-    size = (uintptr_t)get_tboot_mem_end() - base;
+    base = (uint64_t)TBOOT_BASE_ADDR;
+    size = (uint64_t)get_tboot_mem_end() - base;
     uint32_t mem_type = is_kernel_linux() ? E820_RESERVED : E820_UNUSABLE;
     printk("protecting tboot (%Lx - %Lx) in e820 table\n", base,
            (base + size - 1));
@@ -224,7 +227,7 @@ static void post_launch(void)
      */
     memset(&_tboot_shared, 0, PAGE_SIZE);
     _tboot_shared.uuid = (uuid_t)TBOOT_SHARED_UUID;
-    _tboot_shared.version = 5;
+    _tboot_shared.version = 6;
     _tboot_shared.log_addr = (uint32_t)g_log;
     _tboot_shared.shutdown_entry = (uint32_t)shutdown_entry;
     _tboot_shared.tboot_base = (uint32_t)&_start;
@@ -234,6 +237,14 @@ static void post_launch(void)
          key_size != sizeof(_tboot_shared.s3_key) )
         apply_policy(TB_ERR_S3_INTEGRITY);
     _tboot_shared.num_in_wfs = atomic_read(&ap_wfs_count);
+    if ( use_mwait() ) {
+        _tboot_shared.flags |= TB_FLAG_AP_WAKE_SUPPORT;
+        _tboot_shared.ap_wake_trigger = AP_WAKE_TRIGGER_DEF;
+    }
+    else if ( get_tboot_mwait() ) {
+        printk("ap_wake_mwait specified but the CPU doesn't support it.\n");
+    }
+
     print_tboot_shared(&_tboot_shared);
 
     launch_kernel(true);
@@ -417,7 +428,12 @@ static void shutdown_system(uint32_t shutdown_type)
                 outb(0xcf9, 0x0e);
             }
             else {
-                /* soft reset by writing 0x06 to port 0xcf9 */
+                /* soft reset by writing 0xfe to keyboard reset vector 0x64 */
+                /* BIOSes (that are not performing some special operation, */
+                /* such as update) will turn this into a platform reset as */
+                /* expected. */
+                outb(0x64, 0xfe);
+                /* fall back to soft reset by writing 0x06 to port 0xcf9 */
                 /* (supported by all TXT-capable chipsets) */
                 outb(0xcf9, 0x06);
             }
@@ -455,14 +471,25 @@ static void cap_pcrs(void)
 
 void shutdown(void)
 {
+    static atomic_t ap_in_shutdown;
+
     /* wait-for-sipi only invoked for APs, so skip all BSP shutdown code */
     if ( _tboot_shared.shutdown_type == TB_SHUTDOWN_WFS ) {
+        atomic_inc(&ap_in_shutdown);
+        _tboot_shared.ap_wake_trigger = 0;
         mtx_enter(&ap_lock);
         printk("shutdown(): TB_SHUTDOWN_WFS\n");
-        handle_init_sipi_sipi(get_apicid());
+        if ( use_mwait() )
+            ap_wait(get_apicid());
+        else
+            handle_init_sipi_sipi(get_apicid());
         apply_policy(TB_ERR_FATAL);
     }
 
+    printk("wait until all APs ready for txt shutdown\n");
+    while( atomic_read(&ap_wfs_count) < atomic_read(&ap_in_shutdown) )
+        cpu_relax();
+
     /* ensure localities 0, 1 are inactive (in case kernel used them) */
     release_locality(0);
     release_locality(1);
@@ -478,9 +505,12 @@ void shutdown(void)
         /* create and seal memory integrity measurement */
         if ( !seal_post_k_state() )
             apply_policy(TB_ERR_S3_INTEGRITY);
-
-        /* wipe S3 key from memory now that it is sealed */
-        memset(_tboot_shared.s3_key, 0, sizeof(_tboot_shared.s3_key));
+            /* OK to leave key in memory on failure since if user cared they
+               would have policy that doesn't continue for TB_ERR_S3_INTEGRITY
+               error */
+        else
+            /* wipe S3 key from memory now that it is sealed */
+            memset(_tboot_shared.s3_key, 0, sizeof(_tboot_shared.s3_key));
     }
 
     /* cap dynamic PCRs extended as part of launch (17, 18, ...) */
@@ -499,26 +529,29 @@ void shutdown(void)
         /* we don't have any secrets to scrub, however */
         ;
 
-        /* force APs to exit mini-guests if any are in and wait until all */
-        /* are out before shutting down TXT */
-        printk("waiting for APs (%u) to exit guests...\n",
-               atomic_read(&ap_wfs_count));
-        force_aps_exit();
-        uint32_t timeout = AP_GUEST_EXIT_TIMEOUT;
-        do {
-            if ( timeout % 0x8000 == 0 )
-                printk(".");
+        /* in mwait "mode", APs will be in MONITOR/MWAIT and can be left there */
+        if ( !use_mwait() ) {
+            /* force APs to exit mini-guests if any are in and wait until */
+            /* all are out before shutting down TXT */
+            printk("waiting for APs (%u) to exit guests...\n",
+                   atomic_read(&ap_wfs_count));
+            force_aps_exit();
+            uint32_t timeout = AP_GUEST_EXIT_TIMEOUT;
+            do {
+                if ( timeout % 0x8000 == 0 )
+                    printk(".");
+                else
+                    cpu_relax();
+                if ( timeout % 0x200000 == 0 )
+                    printk("\n");
+                timeout--;
+            } while ( ( atomic_read(&ap_wfs_count) > 0 ) && timeout > 0 );
+            printk("\n");
+            if ( timeout == 0 )
+                printk("AP guest exit loop timed-out\n");
             else
-                cpu_relax();
-            if ( timeout % 0x200000 == 0 )
-                printk("\n");
-            timeout--;
-        } while ( ( atomic_read(&ap_wfs_count) > 0 ) && timeout > 0 );
-        printk("\n");
-        if ( timeout == 0 )
-            printk("AP guest exit loop timed-out\n");
-        else
-            printk("all APs exited guests\n");
+                printk("all APs exited guests\n");
+        }
 
         /* turn off TXT (GETSEC[SEXIT]) */
         txt_shutdown();
