@@ -80,7 +80,12 @@ extern long s3_flag;
 
 extern struct mutex ap_lock;
 
+/* MLE/kernel shared data page (in boot.S) */
+extern tboot_shared_t _tboot_shared;
+
 extern void apply_policy(tb_error_t error);
+extern void cpu_wakeup(uint32_t cpuid, uint32_t sipi_vec);
+extern void print_event(const tpm12_pcr_event_t *evt);
 
 /*
  * this is the structure whose addr we'll put in TXT heap
@@ -90,7 +95,7 @@ static __text const mle_hdr_t g_mle_hdr = {
     uuid              :  MLE_HDR_UUID,
     length            :  sizeof(mle_hdr_t),
     version           :  MLE_HDR_VER,
-    entry_point       :  (uint32_t)&_post_launch_entry - TBOOT_BASE_ADDR,
+    entry_point       :  (uint32_t)&_post_launch_entry - TBOOT_START,
     first_valid_page  :  0,
     mle_start_off     :  (uint32_t)&_mle_start - TBOOT_BASE_ADDR,
     mle_end_off       :  (uint32_t)&_mle_end - TBOOT_BASE_ADDR,
@@ -143,6 +148,8 @@ static void print_mle_hdr(const mle_hdr_t *mle_hdr)
 /* 1 ptable = 3 pages and just 1 loop loop for ptable MLE page table */
 /* can only contain 4k pages */
 
+static __mlept uint8_t g_mle_pt[3 * PAGE_SIZE];  /* pgdir ptr + pgdir + ptab = 3 */
+
 static void *build_mle_pagetable(uint32_t mle_start, uint32_t mle_size)
 {
     void *ptab_base;
@@ -165,8 +172,8 @@ static void *build_mle_pagetable(uint32_t mle_start, uint32_t mle_size)
     }
 
     /* place ptab_base below MLE */
-    ptab_size = 3 * PAGE_SIZE;      /* pgdir ptr + pgdir + ptab = 3 */
-    ptab_base = (void *)((mle_start - ptab_size) & PAGE_MASK);
+    ptab_size = sizeof(g_mle_pt);
+    ptab_base = &g_mle_pt;
     memset(ptab_base, 0, ptab_size);
     printk("ptab_size=%x, ptab_base=%p\n", ptab_size, ptab_base);
 
@@ -196,7 +203,7 @@ static void *build_mle_pagetable(uint32_t mle_start, uint32_t mle_size)
  * will go through all modules to find an SINIT that matches the platform
  * (size can be NULL)
  */
-static bool find_platform_sinit_module(multiboot_info_t *mbi, void **base,
+static bool find_platform_sinit_module(const multiboot_info_t *mbi, void **base,
                                        uint32_t *size)
 {
     if ( base != NULL )
@@ -231,7 +238,7 @@ static bool find_platform_sinit_module(multiboot_info_t *mbi, void **base,
     return false;
 }
 
-bool find_lcp_module(multiboot_info_t *mbi, void **base, uint32_t *size)
+bool find_lcp_module(const multiboot_info_t *mbi, void **base, uint32_t *size)
 {
     size_t size2 = 0;
     void *base2 = NULL;
@@ -268,11 +275,71 @@ bool find_lcp_module(multiboot_info_t *mbi, void **base, uint32_t *size)
     return true;
 }
 
+static __data event_log_container_t *g_elog = NULL;
+
+/* should be called after os_mle_data initialized */
+static void *init_event_log(void)
+{
+    os_mle_data_t *os_mle_data = get_os_mle_data_start(get_txt_heap());
+    g_elog = (event_log_container_t *)&os_mle_data->event_log_buffer;
+
+    memcpy((void *)g_elog->signature, EVTLOG_SIGNATURE,
+           sizeof(g_elog->signature));
+    g_elog->container_ver_major = EVTLOG_CNTNR_MAJOR_VER;
+    g_elog->container_ver_minor = EVTLOG_CNTNR_MINOR_VER;
+    g_elog->pcr_event_ver_major = EVTLOG_EVT_MAJOR_VER;
+    g_elog->pcr_event_ver_minor = EVTLOG_EVT_MINOR_VER;
+    g_elog->size = sizeof(os_mle_data->event_log_buffer);
+    g_elog->pcr_events_offset = sizeof(*g_elog);
+    g_elog->next_event_offset = sizeof(*g_elog);
+
+    return (void *)g_elog;
+}
+
+static void init_os_sinit_ext_data(heap_ext_data_element_t* elts)
+{
+    heap_ext_data_element_t* elt = elts;
+    heap_event_log_ptr_elt_t *evt_log;
+
+    evt_log = (heap_event_log_ptr_elt_t *)elt->data;
+    evt_log->event_log_phys_addr = (uint64_t)(unsigned long)init_event_log();
+    elt->type = HEAP_EXTDATA_TYPE_TPM_EVENT_LOG_PTR;
+    elt->size = sizeof(*elt) + sizeof(*evt_log);
+
+    elt = (void *)elt + elt->size;
+    elt->type = HEAP_EXTDATA_TYPE_END;
+    elt->size = sizeof(*elt);
+}
+
+bool evtlog_append(uint8_t pcr, tb_hash_t *hash, uint32_t type)
+{
+    if ( g_elog == NULL )
+        return true;
+
+    tpm12_pcr_event_t *next = (tpm12_pcr_event_t *)
+                              ((void*)g_elog + g_elog->next_event_offset);
+    
+    if ( g_elog->next_event_offset + sizeof(*next) > g_elog->size )
+        return false;
+
+    next->pcr_index = pcr;
+    next->type = type;
+    memcpy(next->digest, hash, sizeof(*hash));
+    next->data_size = 0;
+
+    g_elog->next_event_offset += sizeof(*next) + next->data_size;
+
+    print_event(next);
+    return true;
+}
+
+__data uint32_t g_using_da = 0;
+
 /*
  * sets up TXT heap
  */
 static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit,
-                                 multiboot_info_t *mbi)
+                                 const multiboot_info_t *mbi)
 {
     txt_heap_t *txt_heap;
     uint64_t *size;
@@ -292,26 +359,28 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit,
     size = (uint64_t *)((uint32_t)os_mle_data - sizeof(uint64_t));
     *size = sizeof(*os_mle_data) + sizeof(uint64_t);
     memset(os_mle_data, 0, sizeof(*os_mle_data));
-    os_mle_data->version = 0x02;
-    os_mle_data->mbi = mbi;
+    os_mle_data->version = 3;
+    os_mle_data->mbi = (multiboot_info_t *)(unsigned long)mbi;
     os_mle_data->saved_misc_enable_msr = rdmsr(MSR_IA32_MISC_ENABLE);
 
     /*
      * OS/loader to SINIT data
      */
+    /* check sinit supported os_sinit_data version */
+    uint32_t version = get_supported_os_sinit_data_ver(sinit);
+    if ( version < MIN_OS_SINIT_DATA_VER ) {
+        printk("unsupported OS to SINIT data version(%u) in sinit\n", version);
+        return NULL;
+    }
+    if ( version > MAX_OS_SINIT_DATA_VER )
+        version = MAX_OS_SINIT_DATA_VER;
+
     os_sinit_data_t *os_sinit_data = get_os_sinit_data_start(txt_heap);
     size = (uint64_t *)((uint32_t)os_sinit_data - sizeof(uint64_t));
-    *size = sizeof(*os_sinit_data) + sizeof(uint64_t);
-    memset(os_sinit_data, 0, sizeof(*os_sinit_data));
-    /* we only support versions 4, 5 depending on what SINIT supports (and
-       use v5 if SINIT supports more recent version */
-    if ( get_supported_os_sinit_data_ver(sinit) == 4 ) {
-        os_sinit_data->version = 4;
-        /* need to make size exactly size of v4 fields or SINIT will fail */
-        *size = offsetof(os_sinit_data_t, efi_rsdt_ptr) + sizeof(uint64_t);
-    }
-    else
-        os_sinit_data->version = 5;
+    *size = calc_os_sinit_data_size(version);
+    memset(os_sinit_data, 0, *size);
+    os_sinit_data->version = version;
+
     /* this is phys addr */
     os_sinit_data->mle_ptab = (uint64_t)(unsigned long)ptab_base;
     os_sinit_data->mle_size = g_mle_hdr.mle_end_off - g_mle_hdr.mle_start_off;
@@ -340,7 +409,9 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit,
     /* capabilities : choose monitor wake mechanism first */
     txt_caps_t sinit_caps = get_sinit_capabilities(sinit);
     txt_caps_t caps_mask = { 0 };
-    caps_mask.rlp_wake_getsec = caps_mask.rlp_wake_monitor = 1;
+    caps_mask.rlp_wake_getsec = 1;
+    caps_mask.rlp_wake_monitor = 1;
+    caps_mask.pcr_map_da = 1;
     os_sinit_data->capabilities._raw = MLE_HDR_CAPS & ~caps_mask._raw;
     if ( sinit_caps.rlp_wake_monitor )
         os_sinit_data->capabilities.rlp_wake_monitor = 1;
@@ -356,6 +427,26 @@ static txt_heap_t *init_txt_heap(void *ptab_base, acm_hdr_t *sinit,
      */
     os_sinit_data->capabilities.ecx_pgtbl = 0;
     /* TODO: when tboot supports EFI then set efi_rsdt_ptr */
+    /* capabilities : choose DA/LG */
+    os_sinit_data->capabilities.pcr_map_no_legacy = 1;
+    if ( sinit_caps.pcr_map_da && get_tboot_prefer_da() )
+        os_sinit_data->capabilities.pcr_map_da = 1;
+    else if ( !sinit_caps.pcr_map_no_legacy )
+        os_sinit_data->capabilities.pcr_map_no_legacy = 0;
+    else if ( sinit_caps.pcr_map_da ) {
+        printk("DA is the only supported PCR mapping by SINIT, use it\n");
+        os_sinit_data->capabilities.pcr_map_da = 1;
+    }
+    else {
+        printk("SINIT capabilities are icompatible (0x%x)\n", sinit_caps._raw);
+        return NULL;
+    }
+    g_using_da = os_sinit_data->capabilities.pcr_map_da;
+        
+
+    /* Event log initialization */
+    if ( os_sinit_data->version >= 6 )
+        init_os_sinit_ext_data(os_sinit_data->ext_data_elts);
 
     print_os_sinit_data(os_sinit_data);
 
@@ -457,7 +548,7 @@ bool txt_is_launched(void)
     return sts.senter_done_sts;
 }
 
-tb_error_t txt_launch_environment(multiboot_info_t *mbi)
+tb_error_t txt_launch_environment(const multiboot_info_t *mbi)
 {
     acm_hdr_t *sinit = NULL;
     void *mle_ptab_base;
@@ -630,7 +721,7 @@ bool txt_prepare_cpu(void)
     return true;
 }
 
-tb_error_t txt_post_launch(void)
+void txt_post_launch(void)
 {
     txt_heap_t *txt_heap;
     os_mle_data_t *os_mle_data;
@@ -660,13 +751,19 @@ tb_error_t txt_post_launch(void)
     printk("saved IA32_MISC_ENABLE = 0x%08x\n",
            os_mle_data->saved_misc_enable_msr);
     wrmsr(MSR_IA32_MISC_ENABLE, os_mle_data->saved_misc_enable_msr);
+    if ( use_mwait() ) {
+        /* set MONITOR/MWAIT support */
+        uint64_t misc;
+        misc = rdmsr(MSR_IA32_MISC_ENABLE);
+        misc |= MSR_IA32_MISC_ENABLE_MONITOR_FSM;
+        wrmsr(MSR_IA32_MISC_ENABLE, misc);
+    }
 
     /* restore pre-SENTER MTRRs that were overwritten for SINIT launch */
     restore_mtrrs(&(os_mle_data->saved_mtrr_state));
 
-    /* now, if there was an error, return it */
-    if ( err != TB_ERR_NONE )
-        return err;
+    /* now, if there was an error, apply policy */
+    apply_policy(err);
 
     /* always set the TXT.CMD.SECRETS flag */
     write_priv_config_reg(TXTCR_CMD_SECRETS, 0x01);
@@ -677,8 +774,41 @@ tb_error_t txt_post_launch(void)
     write_priv_config_reg(TXTCR_CMD_OPEN_LOCALITY1, 0x01);
     read_priv_config_reg(TXTCR_E2STS);   /* just a fence, so ignore return */
     printk("opened TPM locality 1\n");
+}
 
-    return TB_ERR_NONE;
+void ap_wait(unsigned int cpuid)
+{
+    if ( cpuid >= NR_CPUS ) {
+        printk("cpuid (%u) exceeds # supported CPUs\n", cpuid);
+        apply_policy(TB_ERR_FATAL);
+        mtx_leave(&ap_lock);
+        return;
+    }
+
+    /* ensure MONITOR/MWAIT support is set */
+    uint64_t misc;
+    misc = rdmsr(MSR_IA32_MISC_ENABLE);
+    misc |= MSR_IA32_MISC_ENABLE_MONITOR_FSM;
+    wrmsr(MSR_IA32_MISC_ENABLE, misc);
+
+    /* this is close enough to entering monitor/mwait loop, so inc counter */
+    atomic_inc(&ap_wfs_count);
+    atomic_inc((atomic_t *)&_tboot_shared.num_in_wfs);
+    mtx_leave(&ap_lock);
+
+    printk("cpu %u mwait'ing\n", cpuid);
+    while ( _tboot_shared.ap_wake_trigger != cpuid ) {
+        cpu_monitor(&_tboot_shared.ap_wake_trigger, 0, 0);
+        mb();
+        if ( _tboot_shared.ap_wake_trigger == cpuid )
+            break;
+        cpu_mwait(0, 0);
+    }
+
+    uint32_t sipi_vec = (uint32_t)_tboot_shared.ap_wake_addr;
+    atomic_dec(&ap_wfs_count);
+    atomic_dec((atomic_t *)&_tboot_shared.num_in_wfs);
+    cpu_wakeup(cpuid, sipi_vec);
 }
 
 void txt_cpu_wakeup(void)
@@ -706,14 +836,17 @@ void txt_cpu_wakeup(void)
     /* restore pre-SENTER IA32_MISC_ENABLE_MSR */
     wrmsr(MSR_IA32_MISC_ENABLE, os_mle_data->saved_misc_enable_msr);
 
-    if (!verify_stm(cpuid))
+    if ( !verify_stm(cpuid) )
         apply_policy(TB_ERR_POST_LAUNCH_VERIFICATION);
 
     /* enable SMIs */
     printk("enabling SMIs on cpu %u\n", cpuid);
     __getsec_smctrl();
 
-    handle_init_sipi_sipi(cpuid);
+    if ( use_mwait() )
+        ap_wait(cpuid);
+    else
+        handle_init_sipi_sipi(cpuid);
 }
 
 tb_error_t txt_protect_mem_regions(void)
